[PATCH 26/41] union-mount: stop lookup when finding a whiteout

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

 



From: Jan Blunck <jblunck@xxxxxxx>

Stop the lookup if we find a whiteout during union path lookup.

Signed-off-by: Jan Blunck <jblunck@xxxxxxx>
Signed-off-by: Valerie Aurora <vaurora@xxxxxxxxxx>
---
 fs/namei.c |   30 ++++++++++++++++++++++--------
 1 files changed, 22 insertions(+), 8 deletions(-)

diff --git a/fs/namei.c b/fs/namei.c
index 8ebbf4f..fb463ac 100644
--- a/fs/namei.c
+++ b/fs/namei.c
@@ -442,10 +442,10 @@ static int __cache_lookup_topmost(struct nameidata *nd, struct qstr *name,
 	path->dentry = dentry;
 	path->mnt = dentry ? nd->path.mnt : NULL;
 
-	if (!dentry || dentry->d_inode)
+	if (!dentry || (dentry->d_inode || d_is_whiteout(dentry)))
 		return !dentry;
 
 	/* look for the first non-negative dentry */
 
 	while (follow_union_down(&nd->path.mnt, &nd->path.dentry)) {
 		dentry = d_hash_and_lookup(nd->path.dentry, name);
@@ -467,7 +467,7 @@ static int __cache_lookup_topmost(struct nameidata *nd, struct qstr *name,
 				goto out_dput;
 		}
 
-		if (dentry->d_inode)
+		if (dentry->d_inode || d_is_whiteout(dentry))
 			goto out_dput;
 
 		dput(dentry);
@@ -505,6 +505,11 @@ static int __cache_lookup_build_union(struct nameidata *nd, struct qstr *name,
 				return 1;
 		}
 
+		if (d_is_whiteout(dentry)) {
+			dput(dentry);
+			break;
+		}
+
 		if (!dentry->d_inode) {
 			dput(dentry);
 			continue;
@@ -716,7 +721,6 @@ out_unlock:
  * type mismatch and whiteouts.
  *
  * FIXME:
- * - handle DT_WHT
  * - handle union stacks in use
  * - handle union stacks mounted upon union stacks
  * - avoid unnecessary allocations of union locks
@@ -731,7 +735,7 @@ static int __real_lookup_topmost(struct nameidata *nd, struct qstr *name,
 	if (err)
 		return err;
 
-	if (path->dentry->d_inode)
+	if (path->dentry->d_inode || d_is_whiteout(path->dentry))
 		return 0;
 
 	while (follow_union_down(&nd->path.mnt, &nd->path.dentry)) {
@@ -747,7 +751,7 @@ static int __real_lookup_topmost(struct nameidata *nd, struct qstr *name,
 		if (err)
 			goto out;
 
-		if (next.dentry->d_inode) {
+		if (next.dentry->d_inode || d_is_whiteout(next.dentry)) {
 			dput(path->dentry);
 			mntget(next.mnt);
 			*path = next;
@@ -790,6 +794,11 @@ static int __real_lookup_build_union(struct nameidata *nd, struct qstr *name,
 		if (err)
 			goto out;
 
+		if (d_is_whiteout(next.dentry)) {
+			dput(next.dentry);
+			break;
+		}
+
 		if (!next.dentry->d_inode) {
 			dput(next.dentry);
 			continue;
@@ -1610,7 +1619,7 @@ static int __hash_lookup_topmost(struct nameidata *nd, struct qstr *name,
 	if (err)
 		return err;
 
-	if (path->dentry->d_inode)
+	if (path->dentry->d_inode || d_is_whiteout(path->dentry))
 		return 0;
 
 	while (follow_union_down(&nd->path.mnt, &nd->path.dentry)) {
@@ -1628,7 +1637,7 @@ static int __hash_lookup_topmost(struct nameidata *nd, struct qstr *name,
 		if (err)
 			goto out;
 
-		if (next.dentry->d_inode) {
+		if (next.dentry->d_inode || d_is_whiteout(next.dentry)) {
 			dput(path->dentry);
 			mntget(next.mnt);
 			*path = next;
@@ -1666,6 +1675,11 @@ static int __hash_lookup_build_union(struct nameidata *nd, struct qstr *name,
 		if (err)
 			goto out;
 
+		if (d_is_whiteout(next.dentry)) {
+			dput(next.dentry);
+			break;
+		}
+
 		if (!next.dentry->d_inode) {
 			dput(next.dentry);
 			continue;
@@ -442,10 +442,10 @@ static int __cache_lookup_topmost(struct nameidata *nd, struct qstr *name,
 		return !dentry;
 
-	/* look for the first non-negative dentry */
+	/* look for the first non-negative or whiteout dentry */
 
 	while (follow_union_down(&nd->path.mnt, &nd->path.dentry)) {
-- 
1.6.3.3

--
To unsubscribe from this list: send the line "unsubscribe linux-fsdevel" in
the body of a message to majordomo@xxxxxxxxxxxxxxx
More majordomo info at  http://vger.kernel.org/majordomo-info.html

[Index of Archives]     [Linux Ext4 Filesystem]     [Union Filesystem]     [Filesystem Testing]     [Ceph Users]     [Ecryptfs]     [AutoFS]     [Kernel Newbies]     [Share Photos]     [Security]     [Netfilter]     [Bugtraq]     [Yosemite News]     [MIPS Linux]     [ARM Linux]     [Linux Security]     [Linux Cachefs]     [Reiser Filesystem]     [Linux RAID]     [Samba]     [Device Mapper]     [CEPH Development]
  Powered by Linux